#!/usr/bin/env python
# coding: utf-8

# # Starry Night
# 
# By Vincent van Gogh - bgEuwDxel93-Pg — Google Arts & Culture, Public Domain, https://commons.wikimedia.org/w/index.php?curid=25498286

# In[1]:


import tkinter as tk
from tkinter import messagebox, filedialog #, simpledialog, colorchooser
from PIL import ImageTk, Image
#import math

class RotateApp:
    def __init__(self):
        self.original_pil_image = None

        self.window = tk.Tk()
        self.window.title('pil_rotate')
        self.window.protocol('WM_DELETE_WINDOW', self.kill_callback)
        self.window.geometry('400x400')

        # Build the menu.
        self.menubar = tk.Menu(self.window)

        self.menu_file = tk.Menu(self.menubar, tearoff=False)
        self.menu_file.add_command(label='Open...', command=self.open, accelerator='Ctrl+O')
        self.menu_file.add_command(label='Save As...', command=self.save_as, accelerator='Ctrl+S')
        self.menu_file.add_separator()
        self.menu_file.add_command(label='Exit', command=self.kill_callback)
        self.menubar.add_cascade(label='File', menu=self.menu_file)
        
        # Set the main menu.
        self.window.config(menu=self.menubar)

        # Bind menu accelerators.
        self.window.bind_all('<Control-o>', self.ctrl_o_pressed)
        self.window.bind_all('<Control-s>', self.ctrl_s_pressed)

        # Build the window surface.
        self.angle_var = tk.IntVar()
        self.angle_var.trace("w", lambda *args : self.rotate())
        scale = tk.Scale(self.window, from_=0, to=359,
            orient=tk.HORIZONTAL, variable=self.angle_var)
        scale.pack(side=tk.TOP, fill=tk.X, expand=False, anchor='w')

        self.canvas = tk.Canvas(self.window, bg='white',
            borderwidth=2, relief=tk.SUNKEN)
        self.canvas.pack(padx=10, pady=(0, 10),
            side=tk.BOTTOM, fill=tk.BOTH, expand=True)

        # Display the window.
        self.window.focus_force()
        self.window.mainloop()

    def kill_callback(self):
        # Destroy the tkinter window.
        self.window.destroy()

    def show_current_image(self):
        if self.current_pil_image == None: return
        self.current_tk_image = ImageTk.PhotoImage(self.current_pil_image)
        self.canvas.delete('all')
        self.display_image = self.canvas.create_image(0, 0,
            anchor=tk.NW, image=self.current_tk_image)

    # File menu.
    def ctrl_o_pressed(self, event):
        self.open()

    def open(self):
        file_types = [('Image Files', '.png .jpg .bmp .gif')]
        filename = tk.filedialog.askopenfilename(
            defaultextension='.png', filetypes=file_types,
            initialdir='.', title='Open File')
        if not filename: return

        try:
            self.original_pil_image = Image.open(filename)

            # Use RGB mode.
            if self.original_pil_image.mode != 'RGB':
                self.original_pil_image = self.original_pil_image.convert(mode='RGB')

            self.current_pil_image = self.original_pil_image.copy()
            self.show_current_image()
            self.rotate()
        except Exception as e:
            messagebox.showinfo('Open Error', f'Error opening image.\n{e}')

    def ctrl_s_pressed(self, event):
        self.save_as()

    def save_as(self):
        file_types = [('Image Files', '.png .jpg .bmp .gif')]
        filename = filedialog.asksaveasfilename(
            defaultextension='.png', filetypes=file_types,
            initialdir='.', title='Save As')
        if not filename: return
        try:
            self.current_pil_image.save(filename)
        except Exception as e:
            messagebox.showinfo('Save Error', f'Error saving image.\n{e}')

    def rotate(self):
        if self.original_pil_image is None: return
        angle = self.angle_var.get()
        self.current_pil_image = self.original_pil_image.rotate(angle)
        self.show_current_image()


# In[2]:


RotateApp()
print('Done')


# In[ ]:




